@@ -288,6 +288,7 @@ typedef enum {
 } tag_type;
 
 typedef struct mod_chxj_config mod_chxj_config;
+typedef struct mod_chxj_req_config_t mod_chxj_req_config;
 
 #if defined(USE_MYSQL_COOKIE)
 #  include "chxj_mysql.h"
@@ -378,6 +379,13 @@ struct mod_chxj_config {
   int                   image_rewrite_mode;
 };
 
+struct mod_chxj_req_config_t {
+  char               *user_agent;
+  device_table       *spec;
+  ap_filter_t        *f;
+  chxjconvrule_entry *entryp;
+};
+
 #define IS_COOKIE_STORE_DBM(X)      ((X) == COOKIE_STORE_TYPE_DBM)
 #define IS_COOKIE_STORE_MYSQL(X)    ((X) == COOKIE_STORE_TYPE_MYSQL)
 #define IS_COOKIE_STORE_MEMCACHE(X) ((X) == COOKIE_STORE_TYPE_MEMCACHE)
@@ -530,6 +538,7 @@ extern char *chxj_node_convert_chxjif_only(
   const char   *src,
   apr_size_t   *len
 );
+extern void chxj_remove_filter(request_rec *r);
 
 #define IMAGE_CACHE_LIMIT_FMT_LEN  (20)
 
